using DotNext;

namespace Common
{
    /// <summary>
    /// 数字处理工具
    /// </summary>
    public class Number
    {
        private static readonly byte[] BitReverseTable = new byte[] {
            0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
            0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
            0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
            0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
            0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
            0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
            0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
            0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
            0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
            0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
            0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
            0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
            0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
            0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
            0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
            0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
            0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
            0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
            0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
            0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
            0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
            0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
            0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
            0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
            0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
            0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
            0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
            0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
            0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
            0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
            0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
            0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
        };

        /// <summary>
        /// 整数转成二进制字节数组
        /// </summary>
        /// <param name="num">整数</param>
        /// <param name="length">整数长度</param>
        /// <param name="isLowNumHigh">是否高位在数组索引低</param>
        /// <returns>二进制字节数组</returns>
        public static Result<byte[]> NumberToBytes(ulong num, uint length, bool isLowNumHigh = false)
        {
            if (length > 8)
            {
                return new(new ArgumentException(
                    "Unsigned long number can't over 8 bytes(64 bits).",
                    nameof(length)
                ));
            }

            var arr = new byte[length];

            if (isLowNumHigh)
            {
                for (var i = 0; i < length; i++)
                {
                    arr[length - 1 - i] = Convert.ToByte((num >> (i << 3)) & (0xFF));
                }
            }
            else
            {
                for (var i = 0; i < length; i++)
                {
                    arr[i] = Convert.ToByte((num >> ((int)(length - 1 - i) << 3)) & (0xFF));
                }
            }
            return arr;
        }

        /// <summary>
        /// 二进制字节数组转成64bits整数
        /// </summary>
        /// <param name="bytes">二进制字节数组</param>
        /// <param name="isLowNumHigh">是否高位在数组索引低</param>
        /// <returns>整数</returns>
        public static Result<UInt64> BytesToUInt64(byte[] bytes, bool isLowNumHigh = false)
        {
            if (bytes.Length > 8)
            {
                return new(new ArgumentException(
                    "Unsigned long number can't over 8 bytes(64 bits).",
                    nameof(bytes)
                ));
            }

            UInt64 num = 0;
            int len = bytes.Length;

            try
            {
                if (isLowNumHigh)
                {
                    for (var i = 0; i < len; i++)
                    {
                        num += Convert.ToUInt64((UInt64)bytes[len - 1 - i] << (i << 3));
                    }
                }
                else
                {
                    for (var i = 0; i < len; i++)
                    {
                        num += Convert.ToUInt64((UInt64)bytes[i] << ((int)(len - 1 - i) << 3));
                    }
                }

                return num;
            }
            catch (Exception error)
            {
                return new(error);
            }
        }

        /// <summary>
        /// 二进制字节数组转成32bits整数
        /// </summary>
        /// <param name="bytes">二进制字节数组</param>
        /// <param name="isLowNumHigh">是否高位在数组索引低</param>
        /// <returns>整数</returns>
        public static Result<UInt32> BytesToUInt32(byte[] bytes, bool isLowNumHigh = false)
        {
            if (bytes.Length > 4)
            {
                return new(new ArgumentException(
                    "Unsigned long number can't over 4 bytes(32 bits).",
                    nameof(bytes)
                ));
            }

            UInt32 num = 0;
            int len = bytes.Length;

            try
            {
                if (isLowNumHigh)
                {
                    for (var i = 0; i < len; i++)
                    {
                        num += Convert.ToUInt32((UInt32)bytes[len - 1 - i] << (i << 3));
                    }
                }
                else
                {
                    for (var i = 0; i < len; i++)
                    {
                        num += Convert.ToUInt32((UInt32)bytes[i] << ((int)(len - 1 - i) << 3));
                    }
                }

                return num;
            }
            catch (Exception error)
            {
                return new(error);
            }

        }

        /// <summary>
        /// [TODO:description]
        /// </summary>
        /// <param name="uintArray">[TODO:parameter]</param>
        /// <returns>[TODO:return]</returns>
        public static Result<byte[]> UInt32ArrayToBytes(UInt32[] uintArray)
        {
            byte[] byteArray = new byte[uintArray.Length * 4];
            try
            {
                Buffer.BlockCopy(uintArray, 0, byteArray, 0, uintArray.Length * 4);
                return byteArray;
            }
            catch (Exception error)
            {
                return new(error);
            }
        }



        /// <summary>
        /// 比特合并成二进制字节
        /// </summary>
        /// <param name="bits1">第一个比特值</param>
        /// <param name="bits1Len">第一个比特值的长度（位数）</param>
        /// <param name="bits2">第二个比特值</param>
        /// <param name="bits2Len">第二个比特值的长度（位数）</param>
        /// <returns>合并后的二进制字节数组</returns>
        public static Result<byte[]> MultiBitsToBytes(ulong bits1, uint bits1Len, ulong bits2, uint bits2Len)
        {
            return NumberToBytes(MultiBitsToNumber(bits1, bits1Len, bits2, bits2Len).Value,
                        (bits1Len + bits2Len) % 8 != 0 ? (bits1Len + bits2Len) / 8 + 1 : (bits1Len + bits2Len) / 8);
        }

        /// <summary>
        /// 比特合并成整型
        /// </summary>
        /// <param name="bits1">第一个比特值</param>
        /// <param name="bits1Len">第一个比特值的长度（位数）</param>
        /// <param name="bits2">第二个比特值</param>
        /// <param name="bits2Len">第二个比特值的长度（位数）</param>
        /// <returns>合并后的整型值</returns>
        public static Result<ulong> MultiBitsToNumber(ulong bits1, uint bits1Len, ulong bits2, uint bits2Len)
        {
            if (bits1Len + bits2Len > 64) return new(new ArgumentException("Two Bits is more than 64 bits"));

            ulong num = (bits1 << Convert.ToInt32(bits2Len)) | bits2;
            return num;
        }

        /// <summary>
        /// 比特合并成整型
        /// </summary>
        /// <param name="bits1">第一个比特值</param>
        /// <param name="bits1Len">第一个比特值的长度（位数）</param>
        /// <param name="bits2">第二个比特值</param>
        /// <param name="bits2Len">第二个比特值的长度（位数）</param>
        /// <returns>合并后的整型值</returns>
        public static Result<uint> MultiBitsToNumber(uint bits1, uint bits1Len, uint bits2, uint bits2Len)
        {
            if (bits1Len + bits2Len > 64) return new(new ArgumentException("Two Bits is more than 64 bits"));

            uint num = (bits1 << Convert.ToInt32(bits2Len)) | bits2;
            return num;
        }

        /// <summary>
        /// 比特位检查
        /// </summary>
        /// <param name="srcBits">源比特值</param>
        /// <param name="dstBits">目标比特值</param>
        /// <param name="mask">掩码（默认为全1）</param>
        /// <returns>检查结果（是否匹配）</returns>
        public static bool BitsCheck(ulong srcBits, ulong dstBits, ulong mask = 0xFFFF_FFFF_FFFF_FFFF)
        {
            return (srcBits & mask) == dstBits;
        }

        /// <summary>
        /// 比特位检查
        /// </summary>
        /// <param name="srcBits">源比特值</param>
        /// <param name="dstBits">目标比特值</param>
        /// <param name="mask">掩码（默认为全1）</param>
        /// <returns>检查结果（是否匹配）</returns>
        public static bool BitsCheck(uint srcBits, uint dstBits, uint mask = 0xFFFF_FFFF)
        {
            return (srcBits & mask) == dstBits;
        }

        /// <summary>
        /// 获取整型对应位置的比特
        /// </summary>
        /// <param name="srcBits">整型数字</param>
        /// <param name="location">位置</param>
        /// <returns>比特</returns>
        public static Result<bool> ToBit(UInt32 srcBits, int location)
        {
            if (location < 0)
                return new(new ArgumentException(
                    "Location can't be negetive", nameof(location)));

            return ((srcBits >> location) & ((UInt32)0b1)) == 1;
        }


        /// <summary>
        /// 字符串转二进制字节数组
        /// </summary>
        /// <param name="str">输入的字符串</param>
        /// <param name="numBase">进制（默认为16进制）</param>
        /// <returns>转换后的二进制字节数组</returns>
        public static byte[] StringToBytes(string str, int numBase = 16)
        {
            var len = str.Length;
            var bytesLen = len / 2;
            var bytes = new byte[bytesLen];

            for (var i = 0; i < bytesLen; i++)
            {
                bytes[i] = Convert.ToByte(str.Substring(i * 2, 2), 16);
            }

            return bytes;
        }

        /// <summary>
        /// 反转字节数组中的子数组
        /// </summary>
        /// <param name="srcBytes">源字节数组</param>
        /// <param name="distance">子数组的长度（反转的步长）</param>
        /// <returns>反转后的字节数组</returns>
        public static Result<byte[]> ReverseBytes(byte[] srcBytes, int distance)
        {
            if (distance <= 0)
                return new(new ArgumentException("Distance can't be negetive", nameof(distance)));

            var srcBytesLen = srcBytes.Length;
            if (distance > srcBytesLen)
                return new(new ArgumentException(
                    "Distance is larger than bytesArray", nameof(distance)));
            if (srcBytesLen % distance != 0)
                return new(new ArgumentException(
                    "The length of bytes can't be divided by distance without reminder", nameof(distance)));

            var dstBytes = new byte[srcBytesLen];
            var buffer = new byte[distance];

            for (int i = 0; i < srcBytesLen; i += distance)
            {
                var end = i + distance;
                buffer = srcBytes[i..end];
                Array.Reverse(buffer);
                Array.Copy(buffer, 0, dstBytes, i, distance);
            }

            return dstBytes;
        }

        /// <summary>
        /// 反转字节内比特顺序（使用查找表的方法）
        /// </summary>
        /// <param name="srcByte">字节</param>
        /// <returns>反转后的字节</returns>
        public static byte ReverseBits(byte srcByte)
        {
            return BitReverseTable[srcByte];
        }

        /// <summary>
        /// 反转字节数组的字节内比特顺序（使用查找表的方法）
        /// </summary>
        /// <param name="srcBytes">字节数组</param>
        /// <returns>反转后的字节字节数组</returns>
        public static byte[] ReverseBits(byte[] srcBytes)
        {
            var bytesLen = srcBytes.Length;
            var dstBytes = new byte[bytesLen];
            for (int i = 0; i < bytesLen; i++)
            {
                dstBytes[i] = BitReverseTable[srcBytes[i]];
            }
            return dstBytes;
        }
    }

    /// <summary>
    /// 字符串处理工具
    /// </summary>
    public class String
    {
        /// <summary>
        /// 反转字符串
        /// </summary>
        /// <param name="s">输入的字符串</param>
        /// <returns>反转后的字符串</returns>
        public static string Reverse(string s)
        {
            char[] charArray = s.ToCharArray();
            Array.Reverse(charArray);
            return new string(charArray);
        }

    }

}
